
!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:59:16.566'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item delegate |
	delegate := self itemDelegateFor: aModelIndex.
	
	1 to: (self model rowCountFor: aModelIndex) do: [:row |
		1 to: (self model columnCountFor: aModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aModelIndex.
			item := delegate itemFor: index.
			self viewport addMorph: item atIndex: index.
			aBoolean ifTrue: [item fadeIn]]].! !

!UiTreeView methodsFor: 'item delegation' stamp: 'mt 10/10/2010 10:34:00.872'!
itemDelegateFor: aModelIndex

	^ self defaultItemDelegate! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:42:28.991'!
model: aModel

	super model: aModel.
	self reset.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:59:36.167'!
reset
	self viewport clear.
	
	self fillViewportFor: nil animated: false.	
	
	self viewport
		updateLayout;
		updateExtent.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:59:30.202'!
toggleIndex: aModelIndex

	| row |
	
	row := self viewport rowFor: aModelIndex.
	
	row isExpanded
		ifTrue: [
			row items first setProperty: #isExpanded toValue: false.
			self viewport collapseRow: row]
		ifFalse: [
			row items first setProperty: #isExpanded toValue: true.
			self
				fillViewportFor: row firstModelIndex
				animated: true].
	
	self viewport
		updateLayout;
		updateExtent.! !

!UiModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 10:26:18.507'!
hasChildrenFor: aModelIndex

	^ (self rowCountFor: aModelIndex) > 0! !

!UiSimpleItemDelegate methodsFor: 'building' stamp: 'mt 10/10/2010 11:07:52.587'!
itemFor: aModelIndex

	| icon text item offset |
	icon := aModelIndex icon "ifNil: [Morph new extent: 16@16; color: Color transparent]".
	text := (aModelIndex text
		ifNil: [' ' asText asMorph lock]
		ifNotNil: [:t | t asText asMorph lock]).
	item := Morph new
		color: Color transparent;
		height: text height. "for smaller items use font height"
	
	icon ifNotNil: [item addMorphBack: icon asMorph].
	item addMorphBack: text.
	
	"Manual layouting."
	offset := item left + 3.
	item submorphs do: [:m |
		m left: offset.
		offset := offset + m width + 2.
		m top: (item top - (m height - item height // 2))].
	
	"Expand item horizontally to its contents."
	item
		width: item submorphBounds width + 10; "font rendering bugfix"
		height: item height + 1. "font rendering bugfix"
		
	^ item! !

!UiViewInfo methodsFor: 'initialization' stamp: 'mt 10/10/2010 10:37:32.512'!
clear

	items removeAll.
	size := nil.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 10:29:55.411'!
addMorph: aMorph atIndex: aModelIndex

	| col row indent |	
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: false.
	
	row := self addRowFor: aModelIndex.
	row items add: aMorph.
	
	col := self columnFor: aModelIndex.
	col items add: aMorph beforeIndex: (rows indexOf: row).
	
	self addMorphBack: aMorph.
	
	"Correct indent."
	indent := 0.
	(aModelIndex column = 1) ifTrue: [
		(col itemBefore: aMorph)
			ifNil: [indent := self indent] "root"
			ifNotNil: [:item |
				"The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent].
				"The item before is my sibling. Use its indentation."
				(item valueOfProperty: #modelIndex) parent = aModelIndex parent
					ifTrue: [indent := item valueOfProperty: #indent]]].
	aMorph setProperty: #indent toValue: indent.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 10:23:19.658'!
addRowFor: aModelIndex

	| row |
	aModelIndex column = 1
		ifTrue: ["Create new row."
			rows
				ifEmpty: [row := rows add: UiViewRowInfo new]
				ifNotEmpty: [row := rows add: UiViewRowInfo new after: (rows
					reverseDetect: [:r |
						"sibling row"
						r parentModelIndex = aModelIndex parent
						"parent row"
						or: [r firstModelIndex = aModelIndex parent]])]]
		ifFalse: ["Find the row."
			row := rows
				reverseDetect: [:r | r parentModelIndex = aModelIndex parent]].
	
	^ row! !

!UiTreeViewport methodsFor: 'initialization' stamp: 'mt 10/10/2010 10:38:05.429'!
clear

	columns do: [:colInfo | colInfo clear].
	rows removeAll.
	self removeAllMorphs.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 10:54:27.193'!
collapseRow: aRowInfo

	| foundFirst predecessors |
	foundFirst := false.
	predecessors := {aRowInfo} asOrderedCollection.
	rows
		select: [:row |
			(predecessors anySatisfy: [:pred | row parentModelIndex = pred firstModelIndex])
				ifTrue: [predecessors add: row. foundFirst := true]
				ifFalse: [foundFirst ifTrue: [predecessors := #()]. false]]
		thenDo: [:row |
			row items withIndexDo: [:item :index | (columns at: index) items remove: item].
			self removeAllMorphsIn: row items.
			rows remove: row].! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 10:57:51.587'!
selectionColor

	^ selectionColor ifNil: [selectionColor := Color r: 0.721 g: 0.815 b: 0.972]! !

!UiViewRowInfo methodsFor: 'testing' stamp: 'mt 10/10/2010 10:27:44.554'!
isCollapsed

	^ self isExpanded not! !

!UiViewRowInfo methodsFor: 'testing' stamp: 'mt 10/10/2010 10:27:36.67'!
isExpanded

	^ items notEmpty and: [items first valueOfProperty: #isExpanded]! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 11:06:06.005'!
columnCountFor: aParentModelIndex
	"name, childCount, type"
	
	^ 3! !

!UiSmalltalkModel methodsFor: 'accessing - data' stamp: 'mt 10/10/2010 11:06:01.969'!
textFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ p name].
		col = 2 ifTrue: [^ ('<font color="#555555">', p systemCategories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">package</font>' asHtmlText]].
	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
		
		col = 2 ifTrue: [^ ('<font color="#555555">', (SystemOrganization listAtCategoryNamed: cat) size asString, '</font>') asHtmlText].
		
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		col = 1 ifTrue: [^ cat].
		col = 3 ifTrue: [^ '<font color="#555555">system category</font>' asHtmlText]].
	
	class ifNotNil: [:c |
		col = 1 ifTrue: [^ c].
		col = 2 ifTrue: [^ ('<font color="#555555">', (Smalltalk at: c) organization categories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">class</font>' asHtmlText]].

	protocol ifNotNil: [:p |
		| organization |
		organization := aModelIndex valueOfProperty: #organization.
		col = 1 ifTrue: [^ p].
		col = 2 ifTrue: [^ ('<font color="#555555">', (organization listAtCategoryNamed: p) size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">protocol</font>' asHtmlText]].
	
	method ifNotNil: [:m |
		col = 1 ifTrue: [^ '#', m].
		col = 3 ifTrue: [^ '<font color="#555555">method</font>' asHtmlText]].
		
	^ nil! !
