
!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/9/2010 18:30:20.347'!
reset

	| index item delegate |
	delegate := self defaultItemDelegate.
	
	1 to: (self model rowCountFor: nil) do: [:row |
		1 to: (self model columnCountFor: nil) do: [:col |
			index := self model indexRow: row column: col parent: nil.
			item := delegate itemFor: index.
			self viewport addMorph: item atIndex: index]].
	
	self viewport
		updateLayout;
		updateExtent.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/9/2010 20:11:53.213'!
toggleIndex: aModelIndex

	| parentIndex index item delegate |
	delegate := self defaultItemDelegate.
	
	"Only use parents from the first column."
	parentIndex := (self viewport rowFor: aModelIndex) firstModelIndex.
	
	1 to: (self model rowCountFor: parentIndex) do: [:row |
		1 to: (self model columnCountFor: parentIndex) do: [:col |
			index := self model indexRow: row column: col parent: parentIndex.
			item := delegate itemFor: index.
			self viewport addMorph: item atIndex: index.
			item fadeIn.]].
	
	self viewport
		updateLayout;
		updateExtent.! !

!UiModel methodsFor: 'as yet unclassified' stamp: 'mt 10/8/2010 13:59:48.112'!
hasChildrenFor: aModelIndex! !

!UiSimpleItemDelegate methodsFor: 'building' stamp: 'mt 10/9/2010 19:55:07.249'!
itemFor: aModelIndex

	| icon text item offset font |
	font := Preferences standardListFont.
	icon := aModelIndex icon "ifNil: [Morph new extent: 16@16; color: Color transparent]".
	text := (aModelIndex text
		ifNil: [' ' asText asMorph lock]
		ifNotNil: [:t | t asText asMorph lock]).
	item := Morph new
		color: Color transparent;
		height: font height.
	
	icon ifNotNil: [item addMorphBack: icon asMorph].
	item addMorphBack: text.
	
	"Manual layouting."
	offset := item left + 3.
	item submorphs do: [:m |
		m left: offset.
		offset := offset + m width + 2.
		m top: (item top - (m height - item height // 2))].
	
	"Expand item horizontally to its contents."
	item
		width: item submorphBounds width + 10;
		height: item height + 1.
		
	^ item! !

!UiViewColumnInfo methodsFor: 'items add/remove' stamp: 'mt 10/9/2010 18:02:51.713'!
addItem: aMorphWithModelIndex beforeIndex: aNumber

	^ items add: aMorphWithModelIndex beforeIndex: aNumber! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/9/2010 20:08:34.038'!
addMorph: aMorph atIndex: aModelIndex

	| col row indent |	
	aMorph setProperty: #modelIndex toValue: aModelIndex.
	aMorph setProperty: #isExpanded toValue: false.
	
	row := self addRowFor: aModelIndex.
	row addItem: aMorph.
	
	col := self columnFor: aModelIndex.
	col addItem: aMorph beforeIndex: (rows indexOf: row).
	
	self addMorphBack: aMorph.
	
	"Correct indent."
	indent := 0.
	(aModelIndex column = 1) ifTrue: [
		(col itemBefore: aMorph)
			ifNil: [indent := self indent] "root"
			ifNotNil: [:item |
				"The item before is my parent. Add additional indentation."
				(item valueOfProperty: #modelIndex) = aModelIndex parent
					ifTrue: [indent := (item valueOfProperty: #indent) + self indent].
				"The item before is my sibling. Use its indentation."
				(item valueOfProperty: #modelIndex) parent = aModelIndex parent
					ifTrue: [indent := item valueOfProperty: #indent]]].
	aMorph setProperty: #indent toValue: indent.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/9/2010 19:58:50.013'!
addRowFor: aModelIndex

	| row |
	aModelIndex column = 1 ifTrue: [
		"New row."
		rows
			ifEmpty: [row := rows add: UiViewRowInfo new]
			ifNotEmpty: [row := rows add: UiViewRowInfo new after: (rows
				reverseDetect: [:r |
					"sibling row"
					r parentModelIndex = aModelIndex parent
					"parent row"
					or: [r firstModelIndex = aModelIndex parent]])]].
	aModelIndex column > 1 ifTrue: [row := rows
		reverseDetect: [:r | r parentModelIndex = aModelIndex parent]].
	
	^ row! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/8/2010 14:38:16.77'!
selectionColor

	^ selectionColor ifNil: [selectionColor := Color r: 0.72 g: 0.72 b: 0.9]! !

!UiViewRowInfo methodsFor: 'items add/remove' stamp: 'mt 10/9/2010 12:35:48.755'!
addItem: aMorphWithModelIndex
	"Items are always sorted according to the column."

	[^ items add: aMorphWithModelIndex]
		ensure: [items sort: [:i1 :i2 |
			(i1 valueOfProperty: #modelIndex) column < (i2 valueOfProperty: #modelIndex) column]]! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/9/2010 17:47:39.056'!
columnCountFor: aParentModelIndex
	"name, childCount, type"
	
	^ 3! !

!UiSmalltalkModel methodsFor: 'as yet unclassified' stamp: 'mt 10/9/2010 17:33:02.631'!
hasChildrenFor: aModelIndex

	self halt.! !

!UiSmalltalkModel methodsFor: 'accessing - data' stamp: 'mt 10/9/2010 17:54:49.564'!
textFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ p name].
		col = 2 ifTrue: [^ ('<font color="#555555">', p systemCategories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">package</font>' asHtmlText]].
	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
		
		col = 2 ifTrue: [^ ('<font color="#555555">', (SystemOrganization listAtCategoryNamed: cat) size asString, '</font>') asHtmlText].
		
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		col = 1 ifTrue: [^ cat].
		col = 3 ifTrue: [^ '<font color="#555555">system category</font>' asHtmlText]].
	
	class ifNotNil: [:c |
		col = 1 ifTrue: [^ c].
		col = 2 ifTrue: [^ ('<font color="#555555">', (Smalltalk at: c) organization categories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">class</font>' asHtmlText]].

	protocol ifNotNil: [:p |
		| organization |
		organization := aModelIndex valueOfProperty: #organization.
		col = 1 ifTrue: [^ p].
		col = 2 ifTrue: [^ ('<font color="#555555">', (organization listAtCategoryNamed: p) size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">protocol</font>' asHtmlText]].
	
	method ifNotNil: [:m |
		col = 1 ifTrue: [^ '#', m].
		col = 3 ifTrue: [^ '<font color="#555555">method</font>' asHtmlText]].
		
	^ nil! !
